import os
from datetime import datetime
import time
import pytweening

def timed_range(start, stop, duration, easing_func=lambda t: t):
    """Generator function for a timed range on an easing function

    Returns a generator that eases from start to stop along the curve specified
    by the easing_func, taking a total duration of duration.
    """
    t_start = datetime.now()
    y_span = stop - start
    t = (datetime.now() - t_start).total_seconds()
    t_scaled = 1 - (duration - t) / duration
    i = easing_func(t_scaled) * y_span + start

    while t_scaled < 1:
        i = easing_func(t_scaled) * y_span + start
        t = (datetime.now() - t_start).total_seconds()
        t_scaled = 1 - (duration - t) / duration
        yield i


class NoBacklightDeviceException(Exception):
    pass

class AcpiBacklightControl(object):
    base_dir = '/sys/class/backlight'


    def __init__(self, device_dir=None, time_sleep=0.01):
        """__init__

        :param dir: The directory containing the acpi files
        :param time_sleep: The time to time.wait() to prevent busy waiting
        """

        # Build the path to the directory for the acpi device. This varies
        # depending on the acpi device e.g. intel display drivers have
        # intel_backlight and others have acpi_video0

        if not device_dir:
            possible_device_dirs = os.listdir(self.base_dir)

            if len(possible_device_dirs) == 0:
                raise NoBacklightDeviceException('Could not find any backlight '
                      'devices in {}'.format(self.base_dir))

            # just default to first possible
            device_dir = possible_device_dirs[0]

        self.dir = os.path.join(self.base_dir, device_dir)
        self.time_sleep = time_sleep

    def __enter__(self):
        self.open()
        return self

    def __exit__(self, *args, **kwargs):
        return self.close()

    def open(self):
        # first store the maximum
        max_file_path = os.path.join(self.dir, 'max_brightness')
        with open(max_file_path, 'r') as max_file:
            self._max = int(max_file.read().strip())

        # now open and keep open the brightness file
        brightness_file_path = os.path.join(self.dir, 'brightness')
        self.brightness_file = open(brightness_file_path, mode='r+')

    def close(self):
        return self.brightness_file.close()

    @property
    def max(self):
        return self._max

    @property
    def brightness(self):
        self.brightness_file.seek(0)
        return int(self.brightness_file.read().strip())

    @brightness.setter
    def brightness(self, new_brightness):
        if new_brightness > self.max:
            raise ValueError('Brightness must be in range 0 to {} for this device'.format(self.max))

        self.brightness_file.seek(0)
        new_brightness = str(int(new_brightness))
        self.brightness_file.write(new_brightness)

        # not sure if I need to flush the file or not...

    def animate(self, new_brightness, duration=0.25, easing_func=pytweening.easeOutCubic):
        """Adjusts backlight brightness to new_brightness with an animation

        Default animation is easeOutCubic over 0.25s duration
        """
        anim_range = timed_range(self.brightness, new_brightness, duration, easing_func)
        for i in anim_range:
            self.brightness = i
            # avoid busy waiting on time in the generator
            time.sleep(self.time_sleep)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import argparse
import pytweening
from .acpibacklight import AcpiBacklightControl


def backlight_cli():
    parser = argparse.ArgumentParser(description='Change and animate backlight brightness via acpi')
    parser.add_argument(
        'action',
        default='show',
        choices=['show', 'max', 'set', 'inc', 'dec'],
        metavar='action',
        help='The action that executes. One of "show", "max", "set", '
             '"inc", or "dec"'
    )
    parser.add_argument(
        'operand',
        nargs='?',
        type=int,
        help='The operand the action executes on.'
    )
    parser.add_argument(
        '--duration',
        '-d',
        type=float,
        default=0.25,
        help='The duration for the action to take effect over. Only taken '
             'into account on set, inc, and dec operations.'
    )
    parser.add_argument(
        '--easing-function',
        '-e',
        default='easeOutCubic',
        choices=['easeInBack', 'easeInBounce', 'easeInCirc', 'easeInCubic',
            'easeInElastic', 'easeInExpo', 'easeInOutBack', 'easeInOutBounce',
            'easeInOutCirc', 'easeInOutCubic', 'easeInOutElastic',
            'easeInOutExpo', 'easeInOutQuad', 'easeInOutQuart',
            'easeInOutQuint', 'easeInOutSine', 'easeInQuad', 'easeInQuart',
            'easeInQuint', 'easeInSine', 'easeOutBack', 'easeOutBounce',
            'easeOutCirc', 'easeOutCubic', 'easeOutElastic', 'easeOutExpo',
            'easeOutQuad', 'easeOutQuart', 'easeOutQuint', 'easeOutSine',
            'linear'],
        metavar='EASING_FUNC',
        help='The easing function used for animations. This program uses '
             'PyTweening so use the function names from PyTweening e.g. '
             '"easeOutCubic" or "easeInOutQuad".'
    )
    args = parser.parse_args()

    with AcpiBacklightControl() as control:

        if args.action == 'show':
            print(control.brightness)
        elif args.action == 'max':
            print(control.max)

        else:
            ## we will be animating so figure out params and call the function

            # all of these require an operand!
            if not args.operand:
                print(
                    'The {} action requires an operand!'.format(args.action),
                    file=sys.stderr
                )
                parser.print_usage(file=sys.stderr)
                sys.exit(1)


            # dynamically find the easing function or default
            easing_func = pytweening.easeOutCubic
            if hasattr(pytweening, args.easing_function):
                easing_func = getattr(pytweening, args.easing_function)
            else:
                print('No easing function {} found in PyTweening. '
                      'Defaulting to easeOutCubic.'.format(args.easing_function))

            new_brightness = control.brightness
            if args.action == 'set':
                new_brightness = args.operand
            elif args.action == 'inc':
                new_brightness += args.operand
            elif args.action == 'dec':
                new_brightness -= args.operand

            control.animate(
                new_brightness,
                duration=args.duration,
                easing_func=easing_func
            )



import os
from setuptools import setup, find_packages

def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()

setup(
    name='acpibacklight',
    version='0.2.0',
    packages=find_packages(),
    scripts=[],
    install_requires=['PyTweening==1.0.3'],

    entry_points={
        'console_scripts': [
            'acpi-ease-backlight = acpibacklight.cli:backlight_cli'
        ],
    },

    author='Aaron Abbott',
    author_email='aabmass@gmail.com',
    description='Library and script for changing brightness on Linux via acpi. '
                'Allows for easing animations too!',

    keywords='acpi brightness backlight easing animation',
    long_description=read('README.rst'),
)
